@@ -53,7 +53,7 @@
 #define FUSE_USE_VERSION 26
 #include <fuse.h>
 
-#define DEBUG_ENABLED 1
+#define DEBUG_ENABLED 0
 
 #define GET_FILE_HANDLE(fi)     (GUINT_TO_POINTER ((guint) (fi)->fh))
 #define SET_FILE_HANDLE(fi, fh) ((fi)->fh = (guint64) GPOINTER_TO_UINT (fh))
@@ -305,7 +305,7 @@ mount_record_new (GMount *mount)
   
   mount_record->root = g_mount_get_root (mount);
   name = g_object_get_data (G_OBJECT (mount), "g-stable-name");
-  if (name != NULL)
+  if (name != NULL && *name != 0)
     name = g_strdup (name);
   else
     name = g_mount_get_name (mount);
@@ -677,16 +677,18 @@ file_info_get_stat_mode (GFileInfo *file_info)
         break;
     }
 
-  if (!g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ)
-      || g_file_info_get_attribute_uint32 (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))
+  if (file_type == G_FILE_TYPE_DIRECTORY ||
+      !g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ) ||
+      g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))
     unix_mode |= S_IRUSR;
-  if (!g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)
-      || g_file_info_get_attribute_uint32 (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+  if (!g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE) ||
+      g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
     unix_mode |= S_IWUSR;
-  if (!g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE)
-      || g_file_info_get_attribute_uint32 (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))
+  if (file_type == G_FILE_TYPE_DIRECTORY ||
+      !g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE) ||
+      g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))
     unix_mode |= S_IXUSR;
-
+  
   return unix_mode;
 }
 
@@ -708,11 +710,6 @@ getattr_for_file (GFile *file, struct stat *sbuf)
       sbuf->st_uid = daemon_uid;
       sbuf->st_gid = daemon_gid;
 
-      if (g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_UNIX_UID))
-        sbuf->st_uid = file_info_get_attribute_as_uint (file_info, G_FILE_ATTRIBUTE_UNIX_UID);
-      if (g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_UNIX_GID))
-        sbuf->st_gid = file_info_get_attribute_as_uint (file_info, G_FILE_ATTRIBUTE_UNIX_GID);
-
       g_file_info_get_modification_time (file_info, &mod_time);
       sbuf->st_mtime = mod_time.tv_sec;
       sbuf->st_ctime = mod_time.tv_sec;
@@ -777,7 +774,7 @@ vfs_getattr (const gchar *path, struct stat *sbuf)
     {
       /* Mount list */
 
-      sbuf->st_mode = S_IFDIR | 0555;                   /* mode_t    protection */
+      sbuf->st_mode = S_IFDIR | 0500;                   /* mode_t    protection */
       sbuf->st_nlink = 2 + g_list_length (mount_list);  /* nlink_t   number of hard links */
       sbuf->st_atime = daemon_creation_time;
       sbuf->st_mtime = daemon_creation_time;
@@ -1815,11 +1812,11 @@ vfs_access (const gchar *path, gint mode)
       if (file_info)
         {
           if ((mode & R_OK && (g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ) &&
-                               !g_file_info_get_attribute_uint32 (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))) ||
+                               !g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))) ||
               (mode & W_OK && (g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE) &&
-                               !g_file_info_get_attribute_uint32 (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))) ||
+                               !g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))) ||
               (mode & X_OK && (g_file_info_has_attribute (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE) &&
-                               !g_file_info_get_attribute_uint32 (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))))
+                               !g_file_info_get_attribute_boolean (file_info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE))))
             result = -EACCES;
 
           g_object_unref (file_info);
