@@ -170,11 +170,18 @@ _drive_get_description (HalDevice *d)
       if (hal_device_get_property_bool (d, "storage.cdrom.bdre"))
         second = _("Blu-ray-RE");
       
-      if (second != NULL) {
-        s = g_strdup_printf (_("%s/%s Drive"), first, second);
-      } else {
-        s = g_strdup_printf (_("%s Drive"), first);
-      }
+      if (second != NULL)
+        {
+          /* translators: This wis something like "CD-ROM/DVD Drive" or
+             "CD-RW/Blue-ray Drive" depending on the properties of the drive */
+          s = g_strdup_printf (_("%s/%s Drive"), first, second);
+        }
+      else
+        {
+          /* translators: This wis something like "CD-ROM Drive" or "CD-RW Drive
+             depending on the properties of the drive */
+          s = g_strdup_printf (_("%s Drive"), first);
+        }
     } 
   else if (strcmp (drive_type, "floppy") == 0)
     s = g_strdup (_("Floppy Drive"));
@@ -347,14 +354,14 @@ hal_changed (HalDevice *device, const char *key, gpointer user_data)
 {
   GHalDrive *hal_drive = G_HAL_DRIVE (user_data);
   
-  //g_warning ("volhal modifying %s (property %s changed)", hal_drive->device_path, key);
+  /*g_warning ("volhal modifying %s (property %s changed)", hal_drive->device_path, key);*/
   _update_from_hal (hal_drive, TRUE);
 }
 
 GHalDrive *
 g_hal_drive_new (GVolumeMonitor       *volume_monitor,
-                  HalDevice            *device,
-                  HalPool              *pool)
+                 HalDevice            *device,
+                 HalPool              *pool)
 {
   GHalDrive *drive;
 
@@ -641,7 +648,7 @@ _eject_unmount_mounts_cb (GObject *source_object,
   else
     {
 
-      g_warning ("successfully unmount %p", mount);
+      /*g_warning ("successfully unmounted %p", mount);*/
 
       /* move on to the next mount.. */
       _eject_unmount_mounts (data);
@@ -658,7 +665,7 @@ _eject_unmount_mounts (UnmountMountsOp *data)
   if (data->pending_mounts == NULL)
     {
 
-      g_warning ("all pending mounts done; ejecting drive");
+      /*g_warning ("all pending mounts done; ejecting drive");*/
 
       g_hal_drive_eject_do (data->drive,
                             data->cancellable,
@@ -671,7 +678,7 @@ _eject_unmount_mounts (UnmountMountsOp *data)
       mount = data->pending_mounts->data;
       data->pending_mounts = g_list_remove (data->pending_mounts, mount);
 
-      g_warning ("unmounting %p", mount);
+      /*g_warning ("unmounting %p", mount);*/
 
       g_mount_unmount (mount,
                        data->flags,
@@ -707,7 +714,7 @@ g_hal_drive_eject (GDrive              *drive,
       GMount *mount; /* the mount may be foreign; cannot assume GHalMount */
 
       mount = g_volume_get_mount (G_VOLUME (volume));
-      if (mount != NULL)
+      if (mount != NULL && g_mount_can_unmount (mount))
         data->pending_mounts = g_list_prepend (data->pending_mounts, g_object_ref (mount));
     }
 
@@ -794,7 +801,7 @@ g_hal_drive_poll_for_media (GDrive              *drive,
   data->user_data = user_data;
   data->cancellable = cancellable;
 
-  //g_warning ("Rescanning udi %s", hal_device_get_udi (hal_drive->device));
+  /*g_warning ("Rescanning udi %s", hal_device_get_udi (hal_drive->device));*/
   
   con = hal_pool_get_dbus_connection (hal_drive->pool);
   
@@ -832,10 +839,46 @@ g_hal_drive_poll_for_media_finish (GDrive        *drive,
                                    GAsyncResult  *result,
                                    GError       **error)
 {
-  //g_warning ("poll finish");
+  /*g_warning ("poll finish");*/
   return TRUE;
 }
 
+static char *
+g_hal_drive_get_identifier (GDrive              *drive,
+                             const char          *kind)
+{
+  GHalDrive *hal_drive = G_HAL_DRIVE (drive);
+
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_HAL_UDI) == 0)
+    return g_strdup (hal_device_get_udi (hal_drive->device));
+  
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE) == 0)
+    return g_strdup (hal_drive->device_path);
+  
+  return NULL;
+}
+
+static char **
+g_hal_drive_enumerate_identifiers (GDrive *drive)
+{
+  GHalDrive *hal_drive = G_HAL_DRIVE (drive);
+  GPtrArray *res;
+
+  res = g_ptr_array_new ();
+
+  g_ptr_array_add (res,
+                   g_strdup (G_VOLUME_IDENTIFIER_KIND_HAL_UDI));
+
+  if (hal_drive->device_path && *hal_drive->device_path != 0)
+    g_ptr_array_add (res,
+                     g_strdup (G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE));
+    
+
+  /* Null-terminate */
+  g_ptr_array_add (res, NULL);
+  
+  return (char **)g_ptr_array_free (res, FALSE);
+}
 
 static void
 g_hal_drive_drive_iface_init (GDriveIface *iface)
@@ -853,6 +896,8 @@ g_hal_drive_drive_iface_init (GDriveIface *iface)
   iface->eject_finish = g_hal_drive_eject_finish;
   iface->poll_for_media = g_hal_drive_poll_for_media;
   iface->poll_for_media_finish = g_hal_drive_poll_for_media_finish;
+  iface->get_identifier = g_hal_drive_get_identifier;
+  iface->enumerate_identifiers = g_hal_drive_enumerate_identifiers;
 }
 
 void 
