@@ -53,11 +53,24 @@ struct _GHalMount {
   GIcon *override_icon;
   GFile *override_root;
   gboolean cannot_unmount;
+  gboolean searched_for_icon;
 
   HalDevice *device;
   HalDevice *drive_device;
 };
 
+static GFile *
+_g_find_file_insensitive_finish (GFile        *parent,
+                                 GAsyncResult *result,
+                                 GError      **error);
+
+static void
+_g_find_file_insensitive_async (GFile              *parent,
+                                const gchar        *name,
+                                GCancellable       *cancellable,
+                                GAsyncReadyCallback callback,
+                                gpointer            user_data);
+
 static void g_hal_mount_mount_iface_init (GMountIface *iface);
 
 #define _G_IMPLEMENT_INTERFACE_DYNAMIC(TYPE_IFACE, iface_init)       { \
@@ -126,29 +139,29 @@ g_hal_mount_init (GHalMount *hal_mount)
 }
 
 const static struct {
-        const char *disc_type;
-        const char *icon_name;
-        char *ui_name;
-        char *ui_name_blank;
+  const char *disc_type;
+  const char *icon_name;
+  char *ui_name;
+  char *ui_name_blank;
 } disc_data[] = {
-        {"cd_rom",        "media-optical-cd-rom", N_("CD-ROM Disc"), N_("Blank CD-ROM Disc")},
-        {"cd_r",          "media-optical-cd-r", N_("CD-R Disc"), N_("Blank CD-R Disc")},
-        {"cd_rw",         "media-optical-cd-rw", N_("CD-RW Disc"), N_("Blank CD-RW Disc")},
-        {"dvd_rom",       "media-optical-dvd-rom", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
-        {"dvd_ram",       "media-optical-dvd-ram", N_("DVD-RAM Disc"), N_("Blank DVD-RAM Disc")},
-        {"dvd_r",         "media-optical-dvd-r", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
-        {"dvd_rw",        "media-optical-dvd-rw", N_("DVD-RW Disc"), N_("Blank DVD-RW Disc")},
-        {"dvd_plus_r",    "media-optical-dvd-r-plus", N_("DVD+R Disc"), N_("Blank DVD+R Disc")},
-        {"dvd_plus_rw",   "media-optical-dvd-rw-plus",  N_("DVD+RW Disc"), N_("Blank DVD+RW Disc")},
-        {"dvd_plus_r_dl", "media-optical-dvd-dl-r-plus", N_("DVD+R DL Disc"), N_("Blank DVD+R DL Disc")},
-        {"bd_rom",        "media-optical-bd-rom", N_("Blu-Ray Disc"), N_("Blank Blu-Ray Disc")},
-        {"bd_r",          "media-optical-bd-r", N_("Blu-Ray R Disc"), N_("Blank Blu-Ray R Disc")},
-        {"bd_re",         "media-optical-bd-re", N_("Blu-Ray RW Disc"), N_("Blank Blu-Ray RW Disc")},
-        {"hddvd_rom",     "media-optical-hddvd-rom", N_("HD DVD Disc"), N_("Blank HD DVD Disc")},
-        {"hddvd_r",       "media-optical-hddvd-r", N_("HD DVD-R Disc"), N_("Blank HD DVD-R Disc")},
-        {"hddvd_rw",      "media-optical-hddvd-rw", N_("HD DVD-RW Disc"), N_("Blank HD DVD-RW Disc")},
-        {"mo",            "media-optical-mo", N_("MO Disc"), N_("Blank MO Disc")},
-        {NULL,            "media-optical", N_("Disc"), N_("Blank Disc")},
+  {"cd_rom",        "media-optical-cd-rom", N_("CD-ROM Disc"), N_("Blank CD-ROM Disc")},
+  {"cd_r",          "media-optical-cd-r", N_("CD-R Disc"), N_("Blank CD-R Disc")},
+  {"cd_rw",         "media-optical-cd-rw", N_("CD-RW Disc"), N_("Blank CD-RW Disc")},
+  {"dvd_rom",       "media-optical-dvd-rom", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
+  {"dvd_ram",       "media-optical-dvd-ram", N_("DVD-RAM Disc"), N_("Blank DVD-RAM Disc")},
+  {"dvd_r",         "media-optical-dvd-r", N_("DVD-ROM Disc"), N_("Blank DVD-ROM Disc")},
+  {"dvd_rw",        "media-optical-dvd-rw", N_("DVD-RW Disc"), N_("Blank DVD-RW Disc")},
+  {"dvd_plus_r",    "media-optical-dvd-r-plus", N_("DVD+R Disc"), N_("Blank DVD+R Disc")},
+  {"dvd_plus_rw",   "media-optical-dvd-rw-plus",  N_("DVD+RW Disc"), N_("Blank DVD+RW Disc")},
+  {"dvd_plus_r_dl", "media-optical-dvd-dl-r-plus", N_("DVD+R DL Disc"), N_("Blank DVD+R DL Disc")},
+  {"bd_rom",        "media-optical-bd-rom", N_("Blu-Ray Disc"), N_("Blank Blu-Ray Disc")},
+  {"bd_r",          "media-optical-bd-r", N_("Blu-Ray R Disc"), N_("Blank Blu-Ray R Disc")},
+  {"bd_re",         "media-optical-bd-re", N_("Blu-Ray RW Disc"), N_("Blank Blu-Ray RW Disc")},
+  {"hddvd_rom",     "media-optical-hddvd-rom", N_("HD DVD Disc"), N_("Blank HD DVD Disc")},
+  {"hddvd_r",       "media-optical-hddvd-r", N_("HD DVD-R Disc"), N_("Blank HD DVD-R Disc")},
+  {"hddvd_rw",      "media-optical-hddvd-rw", N_("HD DVD-RW Disc"), N_("Blank HD DVD-RW Disc")},
+  {"mo",            "media-optical-mo", N_("MO Disc"), N_("Blank MO Disc")},
+  {NULL,            "media-optical", N_("Disc"), N_("Blank Disc")},
 };
 
 static const char *
@@ -182,6 +195,142 @@ get_disc_name (const char *disc_type, gboolean is_blank)
     return disc_data[n].ui_name;
 }
 
+typedef struct _MountIconSearchData
+{
+  GHalMount *mount;
+  GFile *root;
+} MountIconSearchData;
+
+static void
+clear_icon_search_data (MountIconSearchData *data)
+{
+  if (data->mount)
+    g_object_unref (data->mount);
+  if (data->root)
+    g_object_unref (data->root);
+  g_free (data);
+}
+
+static void
+on_icon_file_located (GObject *source_object, GAsyncResult *res,
+                      gpointer user_data)
+{
+  GFile *icon_file;
+  GIcon *icon;
+  MountIconSearchData *data = (MountIconSearchData *) (user_data);
+  
+  icon_file = _g_find_file_insensitive_finish (G_FILE (source_object),
+                                               res, NULL);
+  
+  /* TODO: check if the file actually exists? */
+
+  icon = g_file_icon_new (icon_file);
+  g_object_unref (icon_file);
+
+  g_hal_mount_override_icon (data->mount, icon);
+  g_object_unref (icon);
+  
+  clear_icon_search_data (data);
+}
+
+static void
+on_autorun_loaded (GObject *source_object, GAsyncResult *res,
+                   gpointer user_data)
+{
+  gchar *content, *relative_icon_path = NULL;
+  gsize content_length;
+  MountIconSearchData *data = (MountIconSearchData *) (user_data);
+  
+  if (g_file_load_contents_finish (G_FILE (source_object), res, &content,
+                                   &content_length, NULL, NULL))
+    {
+      /* Scan through for an "icon=" line. Can't use GKeyFile,
+       * because .inf files aren't always valid key files
+       **/
+      GRegex *icon_regex;
+      GMatchInfo *match_info;
+      
+      /* [^,] is because sometimes the icon= line
+       * has a comma at the end
+       **/
+      icon_regex = g_regex_new ("icon=([^,\\r\\n]+)",
+                                G_REGEX_CASELESS, 0, NULL);
+      g_regex_match (icon_regex, content, 0,
+                     &match_info);
+      
+      /* Even if there are multiple matches, pick only the
+       * first.
+       **/
+      if (g_match_info_matches (match_info))
+        {
+          gchar *chr;
+          gchar *word = g_match_info_fetch (match_info, 1);
+          
+          /* Replace '\' with '/' */
+          while ((chr = strchr (word, '\\')) != NULL)
+            *chr = '/';
+          
+          /* If the file name's not valid UTF-8,
+           * don't even try to load it
+           **/
+          if (g_utf8_validate (word, -1, NULL))
+            relative_icon_path = word;
+          else
+            g_free (word);
+        }
+      
+      g_match_info_free (match_info);
+      
+      g_regex_unref (icon_regex);
+      g_free (content);
+    }
+  
+  if (relative_icon_path)
+    {
+      _g_find_file_insensitive_async (data->root,
+                                      relative_icon_path,
+                                      NULL, on_icon_file_located,
+                                      data);
+      
+      g_free (relative_icon_path);
+    }
+  else
+    clear_icon_search_data (data);
+}
+
+static void
+on_autorun_located (GObject *source_object, GAsyncResult *res,
+                    gpointer user_data)
+{
+  GFile *autorun_path;
+  MountIconSearchData *data = (MountIconSearchData *) (user_data);
+  
+  autorun_path = _g_find_file_insensitive_finish (G_FILE (source_object),
+                                                  res, NULL);
+  if (autorun_path)
+    g_file_load_contents_async (autorun_path, NULL, on_autorun_loaded, data);
+  else
+    clear_icon_search_data (data);
+  
+  g_object_unref (autorun_path);
+}
+
+static void
+_g_find_mount_icon (GHalMount *m)
+{
+  MountIconSearchData *search_data;
+  
+  m->searched_for_icon = TRUE;
+	
+  search_data = g_new (MountIconSearchData, 1);
+  search_data->mount = g_object_ref (m);
+  search_data->root = g_mount_get_root (G_MOUNT (m));
+  
+  _g_find_file_insensitive_async (search_data->root,
+                                  "autorun.inf",
+                                  NULL, on_autorun_located,
+                                  search_data);
+}
 
 #define KILOBYTE_FACTOR 1000.0
 #define MEGABYTE_FACTOR (1000.0 * 1000.0)
@@ -246,22 +395,24 @@ do_update_from_hal (GHalMount *m)
   volume_disc_is_blank = hal_device_get_property_bool (volume, "volume.disc.is_blank");
   volume_disc_type = hal_device_get_property_string (volume, "volume.disc.type");
   
-  //g_warning ("drive_type='%s'", drive_type);
-  //g_warning ("drive_bus='%s'", drive_bus);
-  //g_warning ("drive_uses_removable_media=%d", drive_uses_removable_media);
+  /*g_warning ("drive_type='%s'", drive_type); */
+  /*g_warning ("drive_bus='%s'", drive_bus); */
+  /*g_warning ("drive_uses_removable_media=%d", drive_uses_removable_media); */
   
-  if (strcmp (drive_type, "disk") == 0) {
-    if (strcmp (drive_bus, "ide") == 0)
-      icon_name = "drive-harddisk-ata";
-    else if (strcmp (drive_bus, "scsi") == 0)
-      icon_name = "drive-harddisk-scsi";
-    else if (strcmp (drive_bus, "ieee1394") == 0)
-      icon_name = "drive-harddisk-ieee1394";
-    else if (strcmp (drive_bus, "usb") == 0)
-      icon_name = "drive-harddisk-usb";
-    else
-      icon_name = "drive-harddisk";
-  } else if (strcmp (drive_type, "cdrom") == 0)
+  if (strcmp (drive_type, "disk") == 0)
+    {
+      if (strcmp (drive_bus, "ide") == 0)
+        icon_name = "drive-harddisk-ata";
+      else if (strcmp (drive_bus, "scsi") == 0)
+        icon_name = "drive-harddisk-scsi";
+      else if (strcmp (drive_bus, "ieee1394") == 0)
+        icon_name = "drive-harddisk-ieee1394";
+      else if (strcmp (drive_bus, "usb") == 0)
+        icon_name = "drive-harddisk-usb";
+      else
+        icon_name = "drive-harddisk";
+    }
+  else if (strcmp (drive_type, "cdrom") == 0)
     icon_name = g_strdup (get_disc_icon (volume_disc_type));
   else if (strcmp (drive_type, "floppy") == 0)
     icon_name = "media-floppy";
@@ -279,20 +430,22 @@ do_update_from_hal (GHalMount *m)
     icon_name = "drive-harddisk";
 
   
-  if (volume_fs_label != NULL && strlen (volume_fs_label) > 0) {
+  if (volume_fs_label != NULL && strlen (volume_fs_label) > 0)
     name = g_strdup (volume_fs_label);
-  } else if (volume_is_disc) {
-    if (volume_disc_has_audio) {
-      if (volume_disc_has_data)
-        name = g_strdup (_("Mixed Audio/Data Disc"));
+  else if (volume_is_disc)
+    {
+      if (volume_disc_has_audio)
+        {
+          if (volume_disc_has_data)
+            name = g_strdup (_("Mixed Audio/Data Disc"));
+          else
+            name = g_strdup (_("Audio Disc"));
+        }
       else
-        name = g_strdup (_("Audio Disc"));
-    } else {
-      name = g_strdup (get_disc_name (volume_disc_type, volume_disc_is_blank));
+        name = g_strdup (get_disc_name (volume_disc_type, volume_disc_is_blank));
     }
-  } else {
+  else
     name = format_size_for_display (volume_size);
-  }
 
   if (m->override_name != NULL)
     {
@@ -300,18 +453,18 @@ do_update_from_hal (GHalMount *m)
       g_free (name);
     }
   else
-    {
-      m->name = name;
-    }
+    m->name = name;
 
   if (m->override_icon != NULL)
-    {
-      m->icon = g_object_ref (m->override_icon);
-    }
+    m->icon = g_object_ref (m->override_icon);
   else
-    {
-      m->icon = g_themed_icon_new_with_default_fallbacks (icon_name);
-    }
+    m->icon = g_themed_icon_new_with_default_fallbacks (icon_name);
+    
+  /* If this is a CD-ROM, begin searching for an icon specified in
+   * autorun.inf.
+  **/
+  if (strcmp (drive_type, "cdrom") == 0 && !m->searched_for_icon)
+    _g_find_mount_icon (m);
 }
 
 
@@ -351,7 +504,7 @@ hal_changed (HalDevice *device, const char *key, gpointer user_data)
 {
   GHalMount *hal_mount = G_HAL_MOUNT (user_data);
   
-  //g_warning ("mounthal modifying %s (property %s changed)", hal_mount->device_path, key);
+  /*g_warning ("mounthal modifying %s (property %s changed)", hal_mount->device_path, key); */
   update_from_hal (hal_mount, TRUE);
 }
 
@@ -369,18 +522,12 @@ compute_uuid (GHalMount *mount)
   if (strlen (fs_uuid) == 0)
     {
       if (strlen (fs_label) == 0)
-        {
-          mount->uuid = NULL;
-        }
+        mount->uuid = NULL;
       else
-        {
-          mount->uuid = g_strdup (fs_label);
-        }
+        mount->uuid = g_strdup (fs_label);
     }
   else
-    {
-      mount->uuid = g_strdup (fs_uuid);
-    }
+    mount->uuid = g_strdup (fs_uuid);
 }
 
 
@@ -402,13 +549,13 @@ g_hal_mount_new_for_hal_device    (GVolumeMonitor    *volume_monitor,
   if (storage_udi == NULL)
     goto fail;
   
-  //g_warning ("storage_udi = %s", storage_udi);
+  /* g_warning ("storage_udi = %s", storage_udi); */
   
   drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
   if (drive_device == NULL)
     goto fail;
   
-  //g_warning ("drive_device = %p", drive_device);
+  /* g_warning ("drive_device = %p", drive_device); */
   
   mount = g_object_new (G_TYPE_HAL_MOUNT, NULL);
   mount->volume_monitor = volume_monitor;
@@ -459,7 +606,7 @@ g_hal_mount_override_icon (GHalMount *mount, GIcon *icon)
   if (mount->override_icon != NULL)
     g_object_unref (mount->override_icon);
 
-  if (icon == NULL)
+  if (icon != NULL)
     mount->override_icon = g_object_ref (icon);
   else
     mount->override_icon = NULL;
@@ -497,22 +644,22 @@ g_hal_mount_new (GVolumeMonitor       *volume_monitor,
                                                              "volume",
                                                              "block.device",
                                                              mount->device_path);
-      //g_warning ("device = %p for %s", device, mount->device_path);
+      /* g_warning ("device = %p for %s", device, mount->device_path); */
       if (device != NULL)
         {      
-          //g_warning ("udi = %s", hal_device_get_udi (device));
-
+          /* g_warning ("udi = %s", hal_device_get_udi (device)); */
+          
           storage_udi = hal_device_get_property_string (device, "block.storage_device");
           if (storage_udi == NULL)
             goto not_hal;
 
-          //g_warning ("storage_udi = %s", storage_udi);
+          /* g_warning ("storage_udi = %s", storage_udi); */
           
           drive_device = hal_pool_get_device_by_udi (pool, storage_udi);
           if (drive_device == NULL)
             goto not_hal;
 
-          //g_warning ("drive_device = %p", drive_device);
+          /* g_warning ("drive_device = %p", drive_device); */
           
           mount->device = g_object_ref (device);
           mount->drive_device = g_object_ref (drive_device);
@@ -791,18 +938,19 @@ unmount_do (GMount               *mount,
                                  NULL,           /* standard_input */
                                  NULL,           /* standard_output */
                                  &(data->error_fd),
-                                 &error)) {
-    GSimpleAsyncResult *simple;
-    simple = g_simple_async_result_new_from_error (data->object,
-                                                   data->callback,
-                                                   data->user_data,
-                                                   error);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
-    g_error_free (error);
-    g_free (data);
-    return;
-  }
+                                 &error))
+    {
+      GSimpleAsyncResult *simple;
+      simple = g_simple_async_result_new_from_error (data->object,
+                                                     data->callback,
+                                                     data->user_data,
+                                                     error);
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
+      g_error_free (error);
+      g_free (data);
+      return;
+    }
   data->error_string = g_string_new ("");
   data->error_channel = g_io_channel_unix_new (data->error_fd);
   data->error_channel_source_id = g_io_add_watch (data->error_channel, G_IO_IN, unmount_read_error, data);
@@ -822,9 +970,7 @@ g_hal_mount_unmount (GMount              *mount,
   gboolean using_legacy = FALSE;
 
   if (hal_mount->device != NULL)
-    {
-      argv[4] = hal_mount->device_path;
-    }
+    argv[4] = hal_mount->device_path;
   else
     {
       using_legacy = TRUE;
@@ -924,3 +1070,260 @@ g_hal_mount_register (GIOModule *module)
 {
   g_hal_mount_register_type (G_TYPE_MODULE (module));
 }
+
+#define INSENSITIVE_SEARCH_ITEMS_PER_CALLBACK 100
+ 
+static void
+enumerated_children_callback (GObject *source_object, GAsyncResult *res,
+                              gpointer user_data);
+ 
+static void
+more_files_callback (GObject *source_object, GAsyncResult *res,
+                     gpointer user_data);
+
+typedef struct _InsensitiveFileSearchData
+{
+	GFile *root;
+	gchar *original_path;
+	gchar **split_path;
+	gint index;
+	GFileEnumerator *enumerator;
+	GFile *current_file;
+	
+	GCancellable *cancellable;
+	GAsyncReadyCallback callback;
+	gpointer user_data;
+} InsensitiveFileSearchData;
+
+static void
+_g_find_file_insensitive_async (GFile              *parent,
+                                const gchar        *name,
+                                GCancellable       *cancellable,
+                                GAsyncReadyCallback callback,
+                                gpointer            user_data)
+{
+  InsensitiveFileSearchData *data;
+  
+  g_return_if_fail (g_utf8_validate (name, -1, NULL));
+
+  data = g_new (InsensitiveFileSearchData, 1);
+  data->root = g_object_ref (parent);
+  data->original_path = g_strdup (name);
+  data->split_path = g_strsplit (name, G_DIR_SEPARATOR_S, -1);
+  data->index = 0;
+  data->enumerator = NULL;
+  data->current_file = g_object_ref (parent);
+  data->cancellable = cancellable;
+  data->callback = callback;
+  data->user_data = user_data;
+
+  /* Skip any empty components due to multiple slashes */
+  while (data->split_path[data->index] != NULL &&
+         *data->split_path[data->index] == 0)
+    data->index++;
+  
+  g_file_enumerate_children_async (data->current_file,
+                                   G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                   0, G_PRIORITY_DEFAULT, cancellable,
+                                   enumerated_children_callback, data);
+}
+
+static void
+clear_find_file_insensitive_state (InsensitiveFileSearchData *data)
+{
+  if (data->root)
+    g_object_unref (data->root);
+  if (data->original_path)
+    g_free (data->original_path);
+  if (data->split_path)
+    g_strfreev (data->split_path);
+  if (data->enumerator)
+    g_object_unref (data->enumerator);
+  if (data->current_file)
+    g_object_unref (data->current_file);
+  g_free (data);
+}
+
+static void
+enumerated_children_callback (GObject *source_object, GAsyncResult *res,
+                              gpointer user_data)
+{
+  GFileEnumerator *enumerator;
+  InsensitiveFileSearchData *data = (InsensitiveFileSearchData *) (user_data);
+  
+  enumerator = g_file_enumerate_children_finish (G_FILE (source_object),
+                                                 res, NULL);
+  
+  if (enumerator == NULL)
+    {
+      GSimpleAsyncResult *simple;
+      GFile *file;
+      
+      simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                          data->callback,
+                                          data->user_data,
+                                          _g_find_file_insensitive_async);
+      
+      file = g_file_get_child (data->root, data->original_path);
+      
+      g_simple_async_result_set_op_res_gpointer (simple, g_object_ref (file), g_object_unref);
+      g_simple_async_result_complete_in_idle (simple);
+      g_object_unref (simple);
+      clear_find_file_insensitive_state (data);
+      return;
+    }
+  
+  data->enumerator = enumerator;
+  g_file_enumerator_next_files_async (enumerator,
+                                      INSENSITIVE_SEARCH_ITEMS_PER_CALLBACK,
+                                      G_PRIORITY_DEFAULT,
+                                      data->cancellable,
+                                      more_files_callback,
+                                      data);
+}
+
+static void
+more_files_callback (GObject *source_object, GAsyncResult *res,
+                     gpointer user_data)
+{
+  InsensitiveFileSearchData *data = (InsensitiveFileSearchData *) (user_data);
+  GList *files, *l;
+  gchar *filename = NULL, *component, *case_folded_name,
+    *name_collation_key;
+  gboolean end_of_files;
+  
+  files = g_file_enumerator_next_files_finish (data->enumerator,
+                                               res, NULL);
+  
+  end_of_files = files == NULL;
+  
+  component = data->split_path[data->index];
+  g_return_if_fail (component != NULL);
+  
+  case_folded_name = g_utf8_casefold (component, -1);
+  name_collation_key = g_utf8_collate_key (case_folded_name, -1);
+  g_free (case_folded_name);
+  
+  for (l = files; l != NULL; l = l->next)
+    {
+      GFileInfo *info;
+      const gchar *this_name;
+      
+      info = l->data;
+      this_name = g_file_info_get_name (info);
+      
+      if (g_utf8_validate (this_name, -1, NULL))
+        {
+          gchar *case_folded, *key;
+          case_folded = g_utf8_casefold (this_name, -1);
+          key = g_utf8_collate_key (case_folded, -1);
+          g_free (case_folded);
+          
+          if (strcmp (key, name_collation_key) == 0)
+            filename = g_strdup (this_name);
+          g_free (key);
+        }
+      if (filename)
+        break;
+    }
+
+  g_list_foreach (files, (GFunc)g_object_unref, NULL);
+  g_list_free (files);
+  g_free (name_collation_key);
+  
+  if (filename)
+    {
+      GFile *next_file;
+      
+      g_object_unref (data->enumerator);
+      data->enumerator = NULL;
+      
+      /* Set the current file and continue searching */
+      next_file = g_file_get_child (data->current_file, filename);
+      g_free (filename);
+      g_object_unref (data->current_file);
+      data->current_file = next_file;
+      
+      data->index++;
+      /* Skip any empty components due to multiple slashes */
+      while (data->split_path[data->index] != NULL &&
+             *data->split_path[data->index] == 0)
+        data->index++;
+      
+      if (data->split_path[data->index] == NULL)
+       {
+          /* Search is complete, file was found */
+          GSimpleAsyncResult *simple;
+          
+          simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                              data->callback,
+                                              data->user_data,
+                                              _g_find_file_insensitive_async);
+          
+          g_simple_async_result_set_op_res_gpointer (simple, g_object_ref (data->current_file), g_object_unref);
+          g_simple_async_result_complete_in_idle (simple);
+          g_object_unref (simple);
+          clear_find_file_insensitive_state (data);
+          return;
+        }
+      
+      /* Continue searching down the tree */
+      g_file_enumerate_children_async (data->current_file,
+                                       G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                       0, G_PRIORITY_DEFAULT,
+                                       data->cancellable,
+                                       enumerated_children_callback,
+                                       data);
+      return;
+    }
+  
+  if (end_of_files)
+    {
+      /* Could not find the given file, abort the search */
+      GSimpleAsyncResult *simple;
+      GFile *file;
+      
+      g_object_unref (data->enumerator);
+      data->enumerator = NULL;
+      
+      simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                          data->callback,
+                                          data->user_data,
+                                          _g_find_file_insensitive_async);
+      
+      file = g_file_get_child (data->root, data->original_path);
+      g_simple_async_result_set_op_res_gpointer (simple, file, g_object_unref);
+      g_simple_async_result_complete_in_idle (simple);
+      g_object_unref (simple);
+      clear_find_file_insensitive_state (data);
+      return;
+    }
+  
+  /* Continue enumerating */
+  g_file_enumerator_next_files_async (data->enumerator,
+                                      INSENSITIVE_SEARCH_ITEMS_PER_CALLBACK,
+                                      G_PRIORITY_DEFAULT,
+                                      data->cancellable,
+                                      more_files_callback,
+                                      data);
+}
+
+static GFile *
+_g_find_file_insensitive_finish (GFile        *parent,
+                                 GAsyncResult *result,
+                                 GError      **error)
+{
+  GSimpleAsyncResult *simple;
+  GFile *file;
+  
+  g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (result), NULL);
+  
+  simple = G_SIMPLE_ASYNC_RESULT (result);
+  
+  if (g_simple_async_result_propagate_error (simple, error))
+    return NULL;
+  
+  file = G_FILE (g_simple_async_result_get_op_res_gpointer (simple));
+  return g_object_ref (file);
+}
+
